package br.shmem.server;

import java.net.*;
import br.shmem.*;
import java.io.*;

/**
 *
 * @author  crolmos & tminc
 */
public class SignedConnection implements Runnable {
    
    Server server;
    Socket socket;
    ServerCommLayer commLayer;
    HostId remoteId;
    
    public SignedConnection(Server server, Socket socket) {
        this.server = server;
        this.socket = socket;
        commLayer = server.getCommLayer();
    }
    
    public void run() {
        try {
            ObjectInputStream in = new ObjectInputStream(socket.getInputStream());
            URL remoteURL = (URL)in.readObject();
            remoteId = server.getHostId(remoteURL);
            Message m = commLayer.recvMessage(socket, remoteId);
            switch (m.getType()) {
                case Message.LOGOUT:
                    commLayer.logout(socket, remoteURL);
                    break;
                case Message.SHARE:
                    commLayer.share(socket, remoteURL, m.getGlobalName());
                    break;
                case Message.UNSHARE:
                    commLayer.unshare(socket, remoteURL, m.getGlobalName());
                    break;
                case Message.REQUEST:
                    commLayer.request(socket, m.getGlobalName());
                    break;
                case Message.VALIDATE:
                    commLayer.validate(socket, m.getGlobalName(), m.getTimeStamp());
                    break;
                case Message.BEGIN_TRANS:
                    commLayer.beginTrans(socket, remoteURL, m.getGlobalName());
                    break;
                case Message.COMMIT:
                    commLayer.commit(socket, remoteURL, m.getGlobalName());
                    break;
                case Message.ROLLBACK:
                    commLayer.rollback(socket, remoteURL, m.getGlobalName());
                    break;
                case Message.UNLOCK:
                    commLayer.unlock(socket, m.getHostId().getURL(), m.getGlobalName(), m.getTimeStamp());
                    break;
                case Message.IDENTIFY:
                    commLayer.identify(socket, m.getURL());
                    break;
                default:
                    commLayer.sendMessage(socket, Message.createError(m.getType(), "Unrecognized message type."));
            }
        }
        catch(IOException e) {
            System.err.println(e);
            e.printStackTrace();
        }
        catch(HostNotFoundException e) {
            System.err.println(e);
            e.printStackTrace();
        }
        catch(CommLayerException e) {
            System.err.println(e);
            e.printStackTrace();
        }
        catch(ClassNotFoundException e) {
            System.err.println(e);
            e.printStackTrace();
        }
        catch(InsecureObjectException e) {
            System.err.println(e);
            e.printStackTrace();
        }
        finally {
            try {
                socket.close();
            }
            catch(java.io.IOException e) {
            }
        }
    }
}
